# -*- coding: utf-8 -*-
"""
Created on 2014/11/18

@author: nagai
"""

__author__ = 'nagai'

from plone import api
from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from ngi.notify.datadog import _
from ngi.notify.datadog import dd_msg_pool
from ngi.notify.datadog.dd import (metric_datadog,
                                   event_datadog)


class CronDatadog(BrowserView):

    def __call__(self):

        context = self.context
        request = context.REQUEST

        #pool data
        global dd_msg_pool
        if dd_msg_pool:
            for x in dd_msg_pool:
                if x['type'] == 'dd_event':
                    event_datadog(
                        x['title'],
                        x['text'],
                        date_happened=x['date_happened'],
                        tags=x['tags']
                    )
                elif x['type'] == 'dd_metric':
                    metric_datadog(
                        x['metric_name'],
                        x['value'],
                        tags=x['tags']
                    )
            dd_msg_pool = []

        #DB size
        metric_name = 'plone.db_info'
        db_name, value = self.db_info()
        db_tags = dict(db_name=db_name)
        metric_datadog(metric_name, value=value, tags=db_tags)

        return True

    def db_info(self):
        context = aq_inner(self.context)
        cpanel = context.unrestrictedTraverse('/Control_Panel')
        return cpanel.db_name(), float(cpanel.db_size()[0:-1])



from zope.i18nmessageid import MessageFactory

MessageFactory = MessageFactory('ngi.notify.datadog')
_ = MessageFactory

dd_msg_pool = []


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from plone.app.testing import PloneWithPackageLayer
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting

import ngi.notify.datadog


NGI_NOTIFY_DATADOG = PloneWithPackageLayer(
    zcml_package=ngi.notify.datadog,
    zcml_filename='testing.zcml',
    name="NGI_NOTIFY_DATADOG")

NGI_NOTIFY_DATADOG_INTEGRATION = IntegrationTesting(
    bases=(NGI_NOTIFY_DATADOG, ),
    name="NGI_NOTIFY_DATADOG_INTEGRATION")

NGI_NOTIFY_DATADOG_FUNCTIONAL = FunctionalTesting(
    bases=(NGI_NOTIFY_DATADOG, ),
    name="NGI_NOTIFY_DATADOG_FUNCTIONAL")



# encoding: utf-8
"""
Created on 07/18/14

@author: Takashi NAGAI
"""
import time
from datetime import datetime as dt
from plone import api
from ngi.notify.datadog import _
from ngi.notify.datadog import dd_msg_pool
from ngi.notify.datadog.dd import (metric_datadog,
                                   event_datadog)


def createdContent(obj, event):
    """
    Created Event
    """
    user = api.user.get_current()
    path = '/'.join(obj.getPhysicalPath())
    try:
        state = api.content.get_state(obj=obj)
    except:
        state = 'none'
    portal_type = obj.portal_type
    content_type = obj.Type()
    metric_name = 'plone.created'
    tags = dict(user=user.id,
                path=path,
                title=obj.title,
                content_type=content_type,
                portal_type=portal_type,
                workflow=state)
    metric_datadog(metric_name, tags=tags)


def modifiedContent(obj, event):
    """
    Modified Event
    """
    user = api.user.get_current()
    path = '/'.join(obj.getPhysicalPath())
    try:
        state = api.content.get_state(obj=obj)
    except:
        state = 'none'
    portal_type = obj.portal_type
    content_type = obj.Type()
    metric_name = 'plone.modified'
    tags = dict(user=user.id,
                path=path,
                title=obj.title,
                content_type=content_type,
                portal_type=portal_type,
                workflow=state)
    metric_datadog(metric_name, tags=tags)


def actionSucceeded(obj, event):
    """

    :param obj:
    :param event:
    :return:
    """
    #import pdb;pdb.set_trace()
    user = api.user.get_current()
    path = '/'.join(obj.getPhysicalPath())
    try:
        state = api.content.get_state(obj=obj)
    except:
        state = 'none'
    portal_type = obj.portal_type
    content_type = obj.Type()
    wf_action = event.action
    metric_name = 'plone.workflow_action'
    tags = dict(user=user.id,
                path=path,
                title=obj.title,
                content_type=content_type,
                portal_type=portal_type,
                action=wf_action,
                workflow=state)
    metric_datadog(metric_name, tags=tags)


def _log_in_out(metric_name, obj):
    """

    :param metric_name:
    :param obj:
    :return:
    """
    user = api.user.get_current()
    path = '/'.join(obj.getPhysicalPath())
    metric_name = metric_name
    tags = dict(user=user.id,
                path=path)
    metric_datadog(metric_name, tags=tags)


def loggedIn(obj, event):
    """
    logged_out event
    :param obj:
    :param event:
    :return:
    """
    _log_in_out('plone.login', obj)


def loggedOut(obj, event):
    """
    logged_out event
    :param obj:
    :param event:
    :return:
    """
    _log_in_out('plone.logout', obj)


def userCreated(event):
    """

    :param event:
    :return:
    """
    principal = event.principal
    new_userid = principal.getUserId()
    user = api.user.get_current()
    metric_name = 'plone.created_user'
    tags = dict(user=user.id,
                userid=new_userid)
    metric_datadog(metric_name, tags=tags)


def userDeleted(event):
    """

    :param event:
    :return:
    """
    principal = event.principal
    delete_userid = principal
    user = api.user.get_current()
    metric_name = 'plone.deleted_user'
    tags = dict(user=user.id,
                userid=delete_userid)
    metric_datadog(metric_name, tags=tags)


def cpChanged(event):
    """

    :param event:
    :return:
    """
    #import pdb;pdb.set_trace()

    user = api.user.get_current()
    metric_name = 'plone.configuration_changed'
    path = event.context.request.getURL()
    tags = dict(user=user.id,
                path=path)
    metric_datadog(metric_name, tags=tags)


def actionRaisedEvent(obj, event):
    """

    :param obj:
    :param event:
    :return:
    """
    title = _(u'Workflow action error')
    text = _(u'Workflow action error')
    tags = dict()
    event_datadog(
        title,
        text,
        tags=tags
    )


def _start_process(title, text, tags):
    global dd_msg_pool
    now = dt.now()
    date_happened = time.mktime(now.timetuple())
    msg = dict(type=u'dd_event',
               title=title,
               text=text,
               date_happened=date_happened,
               tags=tags)
    dd_msg_pool.append(msg)


def processStart(event):
    """

    :param event:
    :return:
    """
    #import pdb;pdb.set_trace()
    title = _(u"Zope Process Start")
    text = _(u"Zope Process Start")
    tags = {}
    _start_process(title, text, tags)


def databaseOpened(event):
    """

    :param event:
    :return:
    """

    title = _(u"ZODB Opened")
    text = _(u"ZODB Opened")
    tags = {}
    _start_process(title, text, tags)


# encoding: utf-8
"""
Created on 07/18/14

@author: Takashi NAGAI
"""
from zope import schema
from zope.interface import Interface
from plone.app.registry.browser import controlpanel
from ngi.notify.datadog import _


class IDatadog(Interface):
    """ Datadog settings """

    use_dogstatsd = schema.Bool(
        title=_(u'use_dogstatsd_tetxt',
                default=u"Using the DogStatsD"),
        description=_(u"use_dogstatsd_help", default=u'Please check when using the DogStatsD.'),
        default=True,
        required=True)


    statsd_host = schema.TextLine(
        title=_(u'statsd_port_title', default=u'DogStatsD Host'),
        description=_(u"statsd_port_help",
                      default=u'Please enter DogStatsD host when you are using the DogStatsD.'),
        required=False,
        default=u'localhost'
    )

    statsd_port = schema.TextLine(
        title=_(u'statsd_port_title', default=u'DogStatsD Port'),
        description=_(u"statsd_port_help",
                      default=u'Please enter DogStatsD port when you are using the DogStatsD.'),
        required=False,
        default=u'8125'
    )

    api_key = schema.TextLine(
        title=_(u'api_key_title', default=u'DataDog API key'),
        description=_(u"api_key_help",
                      default=u'Please enter API key when you are not using the DogStatsD.'),
        required=False,
    )

    application_key = schema.TextLine(
        title=_(u'application_key_title', default=u'DataDog Application key'),
        description=_(u"application_key_help",
                      default=u'Please enter Application key when you are not using the DogStatsD.'),
        required=False,
    )

    host_name = schema.TextLine(
        title=_(u'host_name_title', default=u'Host Name'),
        description=_(u"host_name_help",
                      default=u'Please enter Host name when you are not using the DogStatsD.'),
        required=False,
    )


class DatadogEditForm(controlpanel.RegistryEditForm):
    """
    Datadog edit form
    """

    schema = IDatadog
    label = _(u'datadog_cp_label', default=u'Notify events to DataDog')
    description = _(u'datadog_cp_help', default=u'Settings')

    def updateWidgets(self):
        super(DatadogEditForm, self).updateWidgets()


class DatadogControlPanel(controlpanel.ControlPanelFormWrapper):
    """
    Datadog Control Panel
    """
    form = DatadogEditForm



import unittest2 as unittest


from ngi.notify.datadog.testing import\
    NGI_NOTIFY_DATADOG_INTEGRATION


class TestExample(unittest.TestCase):

    layer = NGI_NOTIFY_DATADOG_INTEGRATION
    
    def setUp(self):
        # you'll want to use this to set up anything you need for your tests 
        # below
        pass

    def test_success(self):
        sum = 1 + 3
        self.assertEquals(sum, 4)

    def test_failure(self):
        sum = 2 + 3
        self.assertEquals(sum, 4)



# encoding: utf-8
"""
Created on 11/17/14

@author: Takashi NAGAI
"""

__author__ = 'nagai'

import time
import logging
from random import random
from datetime import datetime as dt
from dogapi import dog_http_api
import statsd
from plone import api
try:
    from itertools import imap
except ImportError:
    imap = map
from ngi.notify.datadog import _

logger = logging.getLogger(__name__)


class DogStasd4Plone(statsd.DogStatsd):
    """
    DogStatsd wrapper class
    """

    def _report(self, metric, metric_type, value, tags, sample_rate):
        if sample_rate != 1 and random() > sample_rate:
            return

        payload = [metric, u":", value, u"|", metric_type]
        if sample_rate != 1:
            payload.extend([u"|@", sample_rate])
        if tags:
            payload.extend([u"|#", u",".join(tags)])

        encoded = u"".join(imap(unicode, payload))
        self._send(encoded)


statsd_plone = DogStasd4Plone()


def _get_connect_string():
    """

    :return:
    """
    use_dogstatsd = statsd_host = statsd_port = dd_api_key = dd_app_key = host_name = ''
    try:
        use_dogstatsd = api.portal.get_registry_record('ngi.notify.datadog.controlpanel.IDatadog.use_dogstatsd')
        statsd_host = api.portal.get_registry_record('ngi.notify.datadog.controlpanel.IDatadog.statsd_host')
        statsd_port = api.portal.get_registry_record('ngi.notify.datadog.controlpanel.IDatadog.statsd_port')
        dd_api_key = api.portal.get_registry_record('ngi.notify.datadog.controlpanel.IDatadog.api_key')
        dd_app_key = api.portal.get_registry_record('ngi.notify.datadog.controlpanel.IDatadog.application_key')
        host_name = api.portal.get_registry_record('ngi.notify.datadog.controlpanel.IDatadog.host_name')
    except:
        logging.warning('ngi.notify.datadog:No registry keys')
    return use_dogstatsd, statsd_host, statsd_port, dd_api_key, dd_app_key, host_name


def metric_datadog(metric_name, value=1.0, tags={}):
    """
    post to Datadog service
    :param metric_name:
    :param value:
    :param tags:
    :return:
    """

    use_dogstatsd, statsd_host, statsd_port, dd_api_key, dd_app_key, host_name = _get_connect_string()

    if metric_name:
        dd_tags = [u"{k}:{v}".format(k=k, v=v) for k, v in tags.items()]
        if use_dogstatsd:
            statsd_plone.connect(statsd_host, statsd_port)
            statsd_plone.gauge(metric_name, value, tags=dd_tags)
        elif dd_api_key:
            dog_http_api.api_key = dd_api_key
            dog_http_api.application_key = dd_app_key
            dog_http_api.metric(metric_name, value, host=host_name, tags=dd_tags)


def event_datadog(title, text, date_happened='', tags={}):
    """

    :param title:
    :param text:
    :param date_happened:
    :param tags:
    :return:
    """

    use_dogstatsd, statsd_host, statsd_port, dd_api_key, dd_app_key, host_name = _get_connect_string()

    if not date_happened:
        now = dt.now()
        date_happened = time.mktime(now.timetuple())

    if title and text:
        dd_tags = [u"{k}:{v}".format(k=k, v=v) for k, v in tags.items()]
        if use_dogstatsd:
            statsd_plone.connect(statsd_host, statsd_port)
            statsd_plone.event(title, text, date_happened=date_happened, tags=dd_tags)
        elif dd_api_key:
            dog_http_api.api_key = dd_api_key
            dog_http_api.application_key = dd_app_key
            dog_http_api.event_with_response(title, text, date_happened=date_happened, tags=dd_tags, host=host_name)



from setuptools import setup, find_packages
import os

version = '1.0b1'

long_description = (
    open('README.txt').read()
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    open('CONTRIBUTORS.txt').read()
    + '\n' +
    open('CHANGES.txt').read()
    + '\n')

setup(name=u'ngi.notify.datadog',
      version=version,
      description="Notify the Plone operation event to the DataDog",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Takashi NAGAI',
      author_email='ngi644@gmail.com',
      url='https://github.com/ngi644/ngi.notify.datadog',
      license='gpl',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['ngi', 'ngi.notify'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'plone.app.registry',
          'plone.api',
          'dogapi',
          'dogstatsd-python',
          'zope.processlifetime',
      ],
      extras_require={'test': ['plone.app.testing']},
      entry_points="""
      # -*- Entry points: -*-
  	  [z3c.autoinclude.plugin]
  	  target = plone
      """,
      )
